from OFS.Cache import Cache, CacheManager
from OFS.SimpleItem import SimpleItem
import Globals
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.Image import Pdata

from zLOG import LOG, ERROR

import FSPermissions
import FSMappings

from os import path, sep
import time
import os
import sys

from types import StringType

_marker = []  # Create a new marker object.

_imageObjects = ['File', 'Image']

class CacheException (Exception): pass

class FSCache (Cache):
    _dir = None

    def setDir(self, dir):
        """ Set the dir """
        d = FSMappings.dirs
        if dir is None: dir = d.get('Default')
        else: dir = d.get(dir)
        if dir is None: 
            raise "ValueError", "That directory cannot be found"

        self._dir = dir

    def _fileName(self, ob):
        """ We must base the filename on the getPhysicalPath, 
        otherwise the chances of the object being accessed from two 
        places are to high
        """
        url = ob.getPhysicalPath().join('/')
        fn = os.path.join(self._dir, url)
        # windows fix
        if sep != '/': fn = fn.replace('/', sep)
        return fn

    def ZCache_invalidate(self, ob):
        """ Invalidates the cache entries that apply to ob. """
        fn = self._fileName(ob)
        try:
            if path.exists(fn):
                os.remove(fn)
        except IOError, msg:
            LOG('FSCacheManger', ERROR, 'IOError removing file', error=msg)

    def ZCache_get(self, ob, view_name, keywords, mtime_func, default):
        """ Gets a cache entry """
        return default

    def ZCache_set(self, ob, data, view_name='', keywords=None, mtime_func=None):
        """ Sets a cache entry. """
        fn = self._fileName(ob)

        if ob.meta_type in _imageObjects and data is None:
            # if its a File or an Image, grab the data
            data = ob.data

        # the dir to the file
        if getattr(ob.aq_explicit, 'isPrincipiaFolderish', 0):
                # its a folder, so we'll make it a folder
                fnId = None
                fnDr = fn

        else: 
                # its not a folder so split it
                fnId = fn
                fnDr = path.dirname(fn)

        # write the file
        try:
            if not path.exists(fnDr): 
                os.makedirs(fnDr)

            if fnId:
                fh = open(fnId, 'wb')
                # isinstance wont work on extension class
                if not isinstance(data, StringType):
                    while data is not None:
                        fh.write(data.data)
                        data = data.next
                else:
                    fh.write(data)
                fh.close()

        except IOError, msg:
            LOG('FSCacheManger', ERROR, 'IOError writing file', error=msg)


caches = {}

class FSCacheManager (CacheManager, SimpleItem):
    security = ClassSecurityInfo()

    manage_options = (
        {'label':'Properties', 'action':'manage_main',},
        ) + CacheManager.manage_options + SimpleItem.manage_options

    meta_type = 'FS Cache Manager'

    def __init__(self, id):
        self.id = id
        self.title = ''
        self._dir = None
        self.__cacheid = '%s_%f' % (self.id, time.time())

    ZCacheManager_getCache__roles__ = ()
    def ZCacheManager_getCache(self):
        """ Cache Settings """
        cacheid = self.__cacheid
        try:
            return caches[cacheid]
        except KeyError:
            cache = FSCache()
            cache.setDir(self._dir)
            caches[cacheid] = cache
            return cache

    security.declareProtected(FSPermissions.Manage, 'manage_main')
    manage_main = DTMLFile('dtml/propsFS', globals())

    security.declareProtected(FSPermissions.Manage, 'manage_editProps')
    def manage_editProps(self, title, dirId=None, REQUEST=None):
        """ Changes the cache settings. """
        self.title = str(title)
        # set this on the current dir as well...
        self._dir = dirId
        self.ZCacheManager_getCache().setDir(dirId)

        if REQUEST is not None:
            return self.manage_main(
                self, REQUEST, manage_tabs_message='Properties changed.')

    security.declareProtected(FSPermissions.Manage, 'getSettings')
    def getSettings(self):
        """ Get the settings for the ZMI """
        dirs = [ {'name':k,'dirpath':v} for k, v in FSMappings.dirs.items() ]
        res = {'title':self.title, 'current':self.ZCacheManager_getCache()._dir, 'dirs':dirs}
        return res

Globals.default__class_init__(FSCacheManager)

manage_addFSCacheManagerForm = DTMLFile('dtml/addFS', globals())

def manage_addFSCacheManager(self, id, REQUEST=None):
    """ Adds a FS cache manager to the folder."""
    self._setObject(id, FSCacheManager(id))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
